import hashlib
import os
import os.path
import nbformat
import tornado.web
from git import Git
from notebook.base.handlers import IPythonHandler


class GitGetHandler(IPythonHandler):
    def initialize(self, repo):
        self.repo = repo

    @tornado.web.authenticated
    def get(self):
        path = self.get_argument('path', '')
        id = self.get_argument('id', '')

        if not id and not path:
            self.finish({'id': '', 'last': ''})
            return

        if not id and path:
            sha = hashlib.sha256()
            sha.update(path.encode())
            id = sha.hexdigest()

        last = list(reversed([[x.commit.hexsha, x.commit.authored_date * 1000] + x.name.split('-') for x in self.repo.tags if id in x.name]))
        self.finish({'id': id, 'versions': last})
        return


class GitRestoreHandler(IPythonHandler):
    def initialize(self, repo):
        self.repo = repo

    @tornado.web.authenticated
    def get(self):
        path = self.get_argument('path', '')
        id = self.get_argument('id', '')
        version = int(self.get_argument('version', 0))

        if not id and not path:
            self.finish({'id': '', 'version': -1, 'contents': {}})
            return

        if not id and path:
            sha = hashlib.sha256()
            sha.update(path.encode())
            id = sha.hexdigest()

        try:
            tag = self.repo.tags['%s-%d' % (id, version)]
        except IndexError:
            self.finish({'id': '', 'version': -1, 'contents': {}})
            return

        past = self.repo.tags[-1]

        git = Git(self.repo.working_tree_dir)
        git.checkout(tag)

        path = os.path.join(self.repo.working_tree_dir, id)
        nb = os.path.join(path, 'NOTEBOOK')

        if os.path.exists(path):
            if os.path.exists(nb):
                nb = nbformat.read(nb, 4)
            else:
                nb = {}
        else:
            nb = {}

        self.finish({'id': id, 'version': version, 'nb': nb})

        git.checkout(past)



import os
import os.path
from git import Repo
from functools import partial
from notebook.utils import url_path_join

from .handlers import GitGetHandler, GitRestoreHandler
from .hook import post_save_autocommit_git


def initialize(nb_server_app):
    web_app = nb_server_app.web_app
    base_url = web_app.settings['base_url']

    repo_root = os.path.join(os.path.abspath(os.curdir), '.autoversion')
    if not os.path.exists(repo_root):
        os.mkdir(repo_root)
        repo = Repo.init(repo_root)
    else:
        repo = Repo(repo_root)

    ignore_root = os.path.join(os.path.abspath(os.curdir), '.gitignore')
    if os.path.exists(ignore_root):
        with open(ignore_root, 'r+') as fp:
            add = True
            for line in fp:
                if '.autoversion' in line:
                    add = False
            if add:
                fp.write('\n.autoversion\n')
    else:
        with open(ignore_root, 'w') as fp:
            fp.write('\n.autoversion\n')

    print('Installing jupyterlab_autoversion handler on path %s' % url_path_join(base_url, 'autoversion'))

    context = {'repo': repo}
    handlers = [(url_path_join(base_url, 'autoversion/get'), GitGetHandler, context),
                (url_path_join(base_url, 'autoversion/restore'), GitRestoreHandler, context)]

    return partial(post_save_autocommit_git, repo=repo), handlers



import hashlib
import json
import os
import os.path


def post_save_autocommit_git(repo, model, *args, **kwargs):
    """convert notebooks to Python script after save with nbconvert
    replaces `jupyter notebook --script`
    """
    if model['type'] != 'notebook':
        return

    if model.get('content') and model.get('path'):
        id = model['content']['metadata'].get('autoversion', '')

        if not id:
            sha = hashlib.sha256()
            sha.update(model['path'].encode())
            id = sha.hexdigest()

        path = os.path.join(repo.working_tree_dir, id)

        if not os.path.exists(path):
            os.mkdir(path)

        nb = os.path.join(path, 'NOTEBOOK')
        with open(nb, 'w') as fp:
            fp.write(json.dumps(model['content']))

        last = len([x for x in repo.tags if id in x.name])

        repo.index.add([nb])
        repo.index.commit('%s-%d' % (id, last))
        repo.create_tag('%s-%d' % (id, last))



from notebook.utils import url_path_join


def load_jupyter_server_extension(nb_server_app):
    """
    Called when the extension is loaded.

    Args:
        nb_server_app (NotebookWebApplication): handle to the Notebook webserver instance.
    """
    web_app = nb_server_app.web_app
    base_url = web_app.settings['base_url']
    host_pattern = '.*$'

    backend = nb_server_app.config.get('JupyterLabAutoversion', {}).get('backend', 'git')

    if backend not in ('git', 's3', 'sql'):
        raise Exception('jupyterlab_autoversion backend not recognized: {}'.format(backend))

    if backend == 's3':
        from .storage.s3 import initialize
    elif backend == 'sql':
        from .storage.sql import initialize
    else:
        from .storage.git import initialize

    hook, handlers = initialize(nb_server_app)

    print('Installing jupyterlab_autoversion handler on path %s' % url_path_join(base_url, 'autoversion'))

    web_app.add_handlers(host_pattern, handlers)

    if callable(nb_server_app.contents_manager.pre_save_hook):
        # grab existing hook, and add on top
        temp_hook = nb_server_app.contents_manager.pre_save_hook

        def _hook(*args, **kwargs):
            temp_hook(*args, **kwargs)
            hook(*args, **kwargs)
        nb_server_app.contents_manager.pre_save_hook = _hook
    else:
        nb_server_app.contents_manager.pre_save_hook = hook
